@@ -174,6 +174,7 @@ class ULSound2D(ULSound):
             sound = self.soundfile = sound.lowpass(lowpass, .5)
         device = self.aud_system.device
         self.sound = handle = device.play(sound)
+
         handle.relative = True
         handle.loop_count = loop_count
         self.aud_system.add(self)
@@ -188,18 +189,18 @@ class ULSound2D(ULSound):
 
     @volume.setter
     def volume(self, val):
-        if self.sound:
+        if self.sound and self.sound.status:
             self.sound.volume = val * self.aud_system.volume
         self._volume = val
 
     @property
     def pitch(self):
-        if self.sound:
+        if self.sound and self.sound.status:
             return self.sound.pitch / logic.getTimeScale()
 
     @pitch.setter
     def pitch(self, val):
-        if self.sound:
+        if self.sound and self.sound.status:
             self.sound.pitch = val * logic.getTimeScale()
 
     @property
@@ -225,6 +226,8 @@ class ULSound2D(ULSound):
     def update(self):
         '''TODO: Documentation
         '''
+        if self.volume == 0:
+            return
         handle = self.sound
         if not handle.status:
             self.finished = True
@@ -327,6 +330,10 @@ class ULSound3D(ULSound):
     def update(self):
         '''TODO: Documentation
         '''
+        if self.volume == 0:
+            for i, handle in enumerate(self.handles[1]):
+                handle.volume = 0
+            return
         aud_system = self.aud_system
         speaker = self.speaker
         if not speaker or speaker.invalid:
