@@ -1,27 +1,31 @@
 '''TODO: Documentation
 '''
-from .curves import ULCurve
-from .curves import create_curve  # noqa
-from .curves import set_curve_points  # noqa
+from .lights import ULLight  # noqa
+from .lights import make_unique_light  # noqa
 from .nodetrees import get_geom_socket  # noqa
+from .nodetrees import get_group_socket  # noqa
 from .nodetrees import get_material_socket  # noqa
 from .nodetrees import get_world_socket  # noqa
 from .nodetrees import modify_geom_socket  # noqa
 from .nodetrees import modify_material_socket  # noqa
 from .nodetrees import modify_world_socket  # noqa
 from .nodetrees import set_geom_socket  # noqa
+from .nodetrees import set_group_socket  # noqa
 from .nodetrees import set_material_socket  # noqa
 from .nodetrees import set_world_socket  # noqa
-from .lights import ULLight
-from .lights import make_unique_light
+from .objects import ULCurve  # noqa
+from .objects import controller_brick_status  # noqa
+from .objects import create_curve  # noqa
+from .objects import set_curve_points  # noqa
 from .raycasting import raycast  # noqa
 from .raycasting import raycast_camera  # noqa
 from .raycasting import raycast_face  # noqa
 from .raycasting import raycast_projectile  # noqa
+from .raycasting import raycast_mouse  # noqa
 from .scene import set_scene  # noqa
-from .visuals import draw_cube
-from .visuals import draw_line
-from .visuals import draw_box
+from .visuals import draw_box  # noqa
+from .visuals import draw_cube  # noqa
+from .visuals import draw_line  # noqa
 from bge import logic
 from bge.types import KX_GameObject as GameObject
 from mathutils import Matrix
@@ -260,112 +264,78 @@ def xrot_to(
     rotating_object,
     target_pos,
     front_axis_code=0,
-    speed=0,
-    time_per_frame=1/60
+    factor=1
 ):
     front_vector = LO_AXIS_TO_VECTOR[front_axis_code]
     vec = rotating_object.getVectTo(target_pos)[1]
-    if speed == 0:
-        if front_axis_code >= 3:
-            vec.negate()
-            front_axis_code = front_axis_code - 3
-        if vec.x == vec.y == vec.z == 0:
-            return
-        rotating_object.alignAxisToVect(vec, front_axis_code, 1.0)
-        rotating_object.alignAxisToVect(LO_AXIS_TO_VECTOR[0], 0, 1.0)
+    vec = project_vector3(vec, 1, 2)
+    vec.normalize()
+    front_vector = rotating_object.getAxisVect(front_vector)
+    front_vector = project_vector3(front_vector, 1, 2)
+    signed_angle = vec.angle_signed(front_vector, None)
+    if signed_angle is None:
+        return
+    abs_angle = abs(signed_angle)
+    if abs_angle < 0.01:
         return True
-    else:
-        vec = project_vector3(vec, 1, 2)
-        vec.normalize()
-        front_vector = rotating_object.getAxisVect(front_vector)
-        front_vector = project_vector3(front_vector, 1, 2)
-        signed_angle = vec.angle_signed(front_vector, None)
-        if signed_angle is None:
-            return
-        abs_angle = abs(signed_angle)
-        if abs_angle < 0.01:
-            return True
-        angle_sign = (signed_angle > 0) - (signed_angle < 0)
-        drot = angle_sign * abs_angle * speed * time_per_frame
-        eulers = rotating_object.localOrientation.to_euler()
-        eulers[0] += drot
-        rotating_object.localOrientation = eulers
-        return False
+    angle_sign = (signed_angle > 0) - (signed_angle < 0)
+    drot = angle_sign * abs_angle * clamp(factor)
+    eulers = rotating_object.localOrientation.to_euler()
+    eulers[0] += drot
+    rotating_object.localOrientation = eulers
+    return False
 
 
 def yrot_to(
     rotating_object,
     target_pos,
     front_axis_code=0,
-    speed=0,
-    time_per_frame=1/60
+    factor=1
 ):
     front_vector = LO_AXIS_TO_VECTOR[front_axis_code]
     vec = rotating_object.getVectTo(target_pos)[1]
-    if speed == 0:
-        if front_axis_code >= 3:
-            vec.negate()
-            front_axis_code = front_axis_code - 3
-        if vec.x == vec.y == vec.z == 0:
-            return
-        rotating_object.alignAxisToVect(vec, front_axis_code, 1.0)
-        rotating_object.alignAxisToVect(LO_AXIS_TO_VECTOR[1], 1, 1.0)
+    vec = project_vector3(vec, 2, 0)
+    vec.normalize()
+    front_vector = rotating_object.getAxisVect(front_vector)
+    front_vector = project_vector3(front_vector, 2, 0)
+    signed_angle = vec.angle_signed(front_vector, None)
+    if signed_angle is None:
+        return
+    abs_angle = abs(signed_angle)
+    if abs_angle < 0.01:
         return True
-    else:
-        vec = project_vector3(vec, 2, 0)
-        vec.normalize()
-        front_vector = rotating_object.getAxisVect(front_vector)
-        front_vector = project_vector3(front_vector, 2, 0)
-        signed_angle = vec.angle_signed(front_vector, None)
-        if signed_angle is None:
-            return
-        abs_angle = abs(signed_angle)
-        if abs_angle < 0.01:
-            return True
-        angle_sign = (signed_angle > 0) - (signed_angle < 0)
-        drot = angle_sign * abs_angle * speed * time_per_frame
-        eulers = rotating_object.localOrientation.to_euler()
-        eulers[1] += drot
-        rotating_object.localOrientation = eulers
-        return False
+    angle_sign = (signed_angle > 0) - (signed_angle < 0)
+    drot = angle_sign * abs_angle * clamp(factor)
+    eulers = rotating_object.localOrientation.to_euler()
+    eulers[1] += drot
+    rotating_object.localOrientation = eulers
+    return False
 
 
 def zrot_to(
     rotating_object,
     target_pos,
     front_axis_code=0,
-    speed=0,
-    time_per_frame=1/60
+    factor=1
 ):
     front_vector = LO_AXIS_TO_VECTOR[front_axis_code]
     vec = rotating_object.getVectTo(target_pos)[1]
-    if speed == 0:
-        if front_axis_code >= 3:
-            vec.negate()
-            front_axis_code = front_axis_code - 3
-        if vec.x == vec.y == vec.z == 0:
-            return
-        rotating_object.alignAxisToVect(vec, front_axis_code, 1.0)
-        rotating_object.alignAxisToVect(LO_AXIS_TO_VECTOR[2], 2, 1.0)
+    vec = project_vector3(vec, 0, 1)
+    vec.normalize()
+    front_vector = rotating_object.getAxisVect(front_vector)
+    front_vector = project_vector3(front_vector, 0, 1)
+    signed_angle = vec.angle_signed(front_vector, None)
+    if signed_angle is None:
         return True
-    else:
-        # project in 2d, compute angle diff, set euler rot 2
-        vec = project_vector3(vec, 0, 1)
-        vec.normalize()
-        front_vector = rotating_object.getAxisVect(front_vector)
-        front_vector = project_vector3(front_vector, 0, 1)
-        signed_angle = vec.angle_signed(front_vector, None)
-        if signed_angle is None:
-            return True
-        abs_angle = abs(signed_angle)
-        if abs_angle < 0.01:
-            return True
-        angle_sign = (signed_angle > 0) - (signed_angle < 0)
-        drot = angle_sign * abs_angle * speed * time_per_frame
-        eulers = rotating_object.localOrientation.to_euler()
-        eulers[2] += drot
-        rotating_object.localOrientation = eulers
-        return False
+    abs_angle = abs(signed_angle)
+    if abs_angle < 0.01:
+        return True
+    angle_sign = (signed_angle > 0) - (signed_angle < 0)
+    drot = angle_sign * abs_angle * clamp(factor)
+    eulers = rotating_object.localOrientation.to_euler()
+    eulers[2] += drot
+    rotating_object.localOrientation = eulers
+    return False
 
 
 def rot_to(
@@ -373,32 +343,28 @@ def rot_to(
     rotating_object,
     target_pos,
     front_axis_code,
-    speed = 0,
-    time_per_frame = 0
+    factor=1
 ):
     if rot_axis_index == 0:
         return xrot_to(
             rotating_object,
             target_pos,
             front_axis_code,
-            speed,
-            time_per_frame
+            factor
         )
     elif rot_axis_index == 1:
         return yrot_to(
             rotating_object,
             target_pos,
             front_axis_code,
-            speed,
-            time_per_frame
+            factor
         )
     elif rot_axis_index == 2:
         return zrot_to(
             rotating_object,
             target_pos,
             front_axis_code,
-            speed,
-            time_per_frame
+            factor
         )
 
 
@@ -418,6 +384,30 @@ def load_json_as_dict(filepath):
         raise FileNotFoundError(f'File {filepath} could not be opened!')
 
 
+class MaterialLoader():
+
+    def __init__(self, game_object):
+        self.status = 0.0
+        self.game_object = game_object
+        if len(game_object.blenderObject.material_slots) < 1:
+            debug('MaterialLoader object needs at least one material slot!')
+            return
+        self.materials = [m for m in bpy.data.materials]
+        logic.getCurrentScene().pre_draw.append(self.load_next)
+
+    def load_next(self):
+        next_mat = self.materials.pop()
+        self.game_object.blenderObject.material_slots[0].material = next_mat
+        self.status += 1 / len(bpy.data.materials)
+        if not self.materials:
+            logic.getCurrentScene().pre_draw.remove(self.load_next)
+            self.on_finish()
+
+    def on_finish(self):
+        pass
+
+
+
 ###############################################################################
 # SCENE
 ###############################################################################
@@ -591,7 +581,16 @@ def get_direction(a, b, local=False) -> Vector:
     return d
 
 
-def map_range(value, in_min, in_max, out_min, out_max, clamp=(None, None)):
+def map_range(value: float, in_min: float, in_max: float, out_min: float, out_max: float, clamp=(None, None)) -> float:
+    """Map a value from one range to another.
+    
+    :param `value`: Value to be remapped.
+    :param `in_min`: Lower end of the original range.
+    :param `in_max`: Upper end of the original range.
+    :param `out_min`: Lower end of the new range.
+    :param `out_max`: Upper end of the new range.
+    :param `clamp`: Clamp the modified value.
+    """
     result = (value - in_min) * (out_max - out_min) / (in_max - in_min) + out_min
     if clamp[0] is not None and result < clamp[0]:
         return clamp[0]
@@ -599,7 +598,17 @@ def map_range(value, in_min, in_max, out_min, out_max, clamp=(None, None)):
         return clamp[1]
     return result
 
-def screen_to_world(x=None, y=None, distance=10):
+
+def screen_to_world(x=None, y=None, distance=10) -> Vector:
+    """Get the world coordinates of a point on the screen in a given distance.
+    
+    :param `x`: X position on the screen. Leave at `None` to use mouse position.
+    :param `y`: Y position on the screen. Leave at `None` to use mouse position.
+    :param `distance`: The distance from the camera at which to get the position.
+    
+    :returns: Position as `Vector`
+    """
+
     camera = logic.getCurrentScene().active_camera
     mouse = logic.mouse
     x = x if x else mouse.position[0]
@@ -609,7 +618,7 @@ def screen_to_world(x=None, y=None, distance=10):
     aim = direction * -distance
     return origin + (aim)
 
-def mouse_over(game_object: GameObject):
+def mouse_over(game_object: GameObject) -> bool:
     scene = game_object.scene
     camera = scene.active_camera
     distance = 2.0 * camera.getDistanceTo(game_object)
@@ -620,4 +629,7 @@ def mouse_over(game_object: GameObject):
         mouse_position[1],
         distance
     )
-    return target is game_object
\ No newline at end of file
+    return target is game_object
+
+def get_local(obj, target) -> Matrix:
+    return obj.worldTransform.inverted() @ target
\ No newline at end of file
